/*
Copyright 2015-2022 Ian Jauslin

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

/*
  parse symbolic expressions
*/

#ifndef SYMBOLIC_PARSER_H
#define SYMBOLIC_PARSER_H

#include "types.h"

// parse a symbolic expression from a char_array
int parse_symbolic_expression(Char_Array str, Fields_Table fields, Variables variables, Polynomial* polynomial);
// from char*
int parse_symbolic_expression_str(char* str, Fields_Table fields, Variables variables, Polynomial* polynomial);

// compute the symbol tree from a string
int char_array_to_symbol_tree(Char_Array str, Tree* symbol_tree);
// from char*
int str_to_symbol_tree(char* str, Tree* symbol_tree);

// find matching '<' and '>'
int matching_bracket(Char_Array str, int start);

// resolve a symbol tree to its corresponding polynomial
int resolve_symbol_tree(Tree symbol_tree, Fields_Table fields, Variables variables, Polynomial* output);

#endif
